/* BSD 3-Clause License
 *
 * Copyright © 2008-2019, Jice and the libtcod contributors.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *
 * 1. Redistributions of source code must retain the above copyright notice,
 *    this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright notice,
 *    this list of conditions and the following disclaimer in the documentation
 *    and/or other materials provided with the distribution.
 *
 * 3. Neither the name of the copyright holder nor the names of its
 *    contributors may be used to endorse or promote products derived from
 *    this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 */
#ifndef LIBTCOD_ENGINE_ERROR_H_
#define LIBTCOD_ENGINE_ERROR_H_

#ifdef __cplusplus
#include <exception>
#include <string>
#endif // __cplusplus

#include "../portability.h"

#ifdef __cplusplus
namespace tcod {
/**
 *  Set an error message and return a relevant error code, usually -1.
 *
 *  Used internally.
 */
TCODLIB_FORMAT(1, 2)
int set_errorf(const char* fmt, ...);
int set_error(const char* msg);
int set_error(const std::string& msg);
int set_error(const std::exception& e);
} // namespace tcod
#endif // __cplusplus
/**
 *  Return the last error message.  If there is no error then the string will
 *  have a length of zero.
 *
 *  The error state is thread specific.
 *  \rst
 *  .. versionadded:: 1.12
 *  \endrst
 */
TCODLIB_CAPI const char* TCOD_get_error();
/**
 *  Set an error message and return -1.
 *  \rst
 *  .. versionadded:: 1.12
 *  \endrst
 */
TCODLIB_CAPI int TCOD_set_error(const char*);
#endif // LIBTCOD_ENGINE_ERROR_H_
